--Script: Egg Watch
--Usage: Gives all levels a countdown. All players must finish the level before the watch runs out of time, or else something will happen.

local eggWatchEnabled = CV_RegisterVar({
	name = "eggwatch",
	defaultvalue = 0,
	flags = CV_SHOWMODIF|CV_NETVAR,
	PossibleValue = CV_OnOff
})

local eggWatchTime = CV_RegisterVar({
	name = "eggwatch_time",
	defaultvalue = 200,
	flags = CV_SHOWMODIF|CV_NETVAR,
	PossibleValue = CV_UNSIGNED
})

local eggwatchPunish = 1

local eggwatchEvent = 1

local RNG = 0

addHook("MapLoad",function()
	if eggWatchEnabled.value == 0 then
		print("\x85 Egg Watch is off.")
	else
		print("\x83 Egg Watch is on.")
		print("\x82(Egg Watch) Time is " .. eggWatchTime.value .. " seconds")
	end
end)

COM_AddCommand("eggwatch_punishment",function(player,args)
    if args then
        if not tonumber(args)
            CONS_Printf(player,"Your choice must be a number from 0 - 5.")
        elseif tonumber(args) < 0
        or tonumber(args) > 3
            CONS_Printf(player,"That is not a valid choice.")
        elseif tonumber(args) == 0 then
            eggwatchPunish = 0
            CONS_Printf(player,"Nothing will happen when the Egg Watch times out.")
        elseif tonumber(args) == 1 then
            eggwatchPunish = 1
            CONS_Printf(player,"Everybody will die when the Egg Watch times out.")
        elseif tonumber(args) == 2 then
            eggwatchPunish = 2
            CONS_Printf(player,"Life as a whole will end when the Egg Watch times out.")
        elseif tonumber(args) == 3 then
            eggwatchPunish = 3
            CONS_Printf(player,"Something will change when the Egg Watch times out. After that, the Egg Watch will reset.")
        end      
    else
        CONS_Printf(player,"eggwatch_punishment - Determines what happens when the Egg Watch runs out of time.")
        CONS_Printf(player,"The values are:")
        CONS_Printf(player,"1 = Everybody in the server dies.")
        CONS_Printf(player,"2 = Every MOBJ in the server dies.")
        CONS_Printf(player,"3 = Something is changed, and the Egg Watch rolls back to its original value.")
    end
end)

COM_AddCommand("eggwatch_event",function(player,args)
    if args then
        if not tonumber(args)
            CONS_Printf(player,"Your choice must be a number from 0 - 8.")
        elseif tonumber(args) < 0
        or tonumber(args) > 8
            CONS_Printf(player,"That is not a valid choice.")
        elseif tonumber(args) == 0 then
            eggwatchEvent = 0
            CONS_Printf(player,"Nothing will happen when the Egg Watch times out.")
        elseif tonumber(args) == 1 then
            eggwatchEvent = 1
            CONS_Printf(player,"Everybody will have a brief identity crisis when the Egg Watch times out.")
        elseif tonumber(args) == 2 then
            eggwatchEvent = 2
            CONS_Printf(player,"People with no Chaos Emeralds will have power over those who do when the Egg Watch times out.")
        elseif tonumber(args) == 3 then
            eggwatchEvent = 3
            CONS_Printf(player,"Everyone will wear Shadow's shoes when the Egg Watch runs out.")
        elseif tonumber(args) == 4 then
            eggwatchEvent = 4
            CONS_Printf(player,"Alice in Wonderland when the Egg Watch runs out.")
        elseif tonumber(args) == 5 then
            eggwatchEvent = 5
            CONS_Printf(player,"Nobody will be able to move or jump when the Egg Watch runs out.")
        elseif tonumber(args) == 6 then
            eggwatchEvent = 6
            CONS_Printf(player,"Nobody can see when the Egg Watch runs out, because nobody can see.")
        elseif tonumber(args) == 7 then
            eggwatchEvent = 7
            CONS_Printf(player,"Every sector's ceiling and floor will close in when the Egg Watch runs out.")
        end                               
    else
        CONS_Printf(player,"eggwatch_event - Determines what happens when the Egg Watch runs out of time on punishment 3.")
        CONS_Printf(player,"The values are:")
        CONS_Printf(player,"0 = Nothing happens.")
        CONS_Printf(player,"1 = Everybody in the server has their skin changed to a random skin.")
        CONS_Printf(player,"2 = Every non-Super player in the server is forced into Super, and every Super player is forced back to normal.")
        CONS_Printf(player,"3 = Everybody loses friction.")
        CONS_Printf(player,"4 = Everybody changes scale.")
        CONS_Printf(player,"5 = Everybody becomes made of concrete.")
        CONS_Printf(player,"6 = Everybody suffers from incurable blindness.")
        CONS_Printf(player,"7 = Every sector's ceiling and floor will close in slightly.")
    end
end)

local eggwatchName = "EGG WATCH"

COM_AddCommand("eggwatch_name",function(player,args)
    if args then
        if not tostring(args) then
            CONS_Printf(player,"The name needs to be a string value.")
        else
            eggwatchName = string.upper(tostring(args))
        end
    else
        CONS_Printf(player,"eggwatch_name - Sets the name of the Egg Watch. By default, it's 'Egg Watch'.")
    end
end)

local timerTics = 0
local eggWatchTimer = 200

local totalRingCount
local tempRingCount

local skinlist = {}
local skinstring = ""

addHook("ThinkFrame", function()
    if eggWatchEnabled.value == 1 then
        if timerTics > 0 then
            timerTics = $ - 1
        end
        if timerTics == 0
        and eggWatchTimer > 0 then
            timerTics = TICRATE
            eggWatchTimer = $ - 1
            if eggWatchTimer == 100
            or eggWatchTimer == 60
            or eggWatchTimer == 30
                S_StartSound(nil,"sfx_3db14")
            end
        end
        if eggWatchTimer == 0 then
            if eggwatchPunish == 1 then
                for player in players.iterate do
                    if player.mo.health > 0 then
                        P_KillMobj(player.mo)
                    end
                end
                if mapheaderinfo[gamemap].forceeggwatchtime then
                    eggWatchTimer = tonumber(mapheaderinfo[gamemap].forceeggwatchtime)
                else
                    eggWatchTimer = eggWatchTime.value
                end
            elseif eggwatchPunish == 2 then
                for thingy in mapthings.iterate do
                    if thingy.mobj then
                        P_KillMobj(thingy.mobj)
                    end
                end
            elseif eggwatchPunish == 3 then
                if eggwatchEvent == 1 then --Change everyone's skins
                    for player in players.iterate do
                        if player.mo and player.mo.valid then
                            local skinRNG = P_RandomRange(0,#skinlist)
                            if R_SkinUsable(player,skinRNG) then
                                R_SetPlayerSkin(player,skinRNG)
                            end
                        end
                    end
                elseif eggwatchEvent == 2 then --Force Non-Supers into Super, and vice-versa
                    for player in players.iterate do
                        if player.mo and player.mo.valid then
                            if player.powers[pw_super] == 0 then
                                player.rings = $ + 50
                                P_SpawnMobj(player.mo.x,player.mo.y,player.mo.z, MT_EMERALD1)
                                P_SpawnMobj(player.mo.x,player.mo.y,player.mo.z, MT_EMERALD2)
                                P_SpawnMobj(player.mo.x,player.mo.y,player.mo.z, MT_EMERALD3)
                                P_SpawnMobj(player.mo.x,player.mo.y,player.mo.z, MT_EMERALD4)
                                P_SpawnMobj(player.mo.x,player.mo.y,player.mo.z, MT_EMERALD5)
                                P_SpawnMobj(player.mo.x,player.mo.y,player.mo.z, MT_EMERALD6)
                                P_SpawnMobj(player.mo.x,player.mo.y,player.mo.z, MT_EMERALD7)
                                player.mo.momz = 0
                                player.mo.momx = 0
                                player.mo.momy = 0
                                player.powers[pw_super] = 1
                                player.mo.state = S_PLAY_SUPER_TRANS1
                            else
                                player.mo.color = player.skincolor
                                player.powers[pw_super] = 0
                                player.rings = P_RandomRange(0,49)
                            end
                        end
                    end
                elseif eggwatchEvent == 3 then --Have everyone become frictionless
                    for player in players.iterate do
                        if player.mo and player.mo.valid then
                            player.mo.ShadowFriction = true
                        end
                    end
                elseif eggwatchEvent == 4 then --Have everyone become various sizes
                    for player in players.iterate do
                        if player.mo and player.mo.valid then
							player.mo.funniscale = (FRACUNIT + P_RandomRange(-FRACUNIT*2,FRACUNIT*2))
                        end
                    end
                elseif eggwatchEvent == 5 then --Have everyone be made of concrete
                    for player in players.iterate do
                        if player.mo and player.mo.valid then
                            player.mo.colorized = true
                            player.mo.color = SKINCOLOR_GREY
                            S_StartSound(nil,28)
                        end
                    end
                elseif eggwatchEvent == 6 then --Lower all sector brightnesses
                    for sector in sectors.iterate do
                        sector.lightlevel = $ - 10
                    end
                elseif eggwatchEvent == 7 then --Make the floor and ceiling close in
                    for sector in sectors.iterate do
                        sector.floorheight = $ + FRACUNIT*5
                        sector.ceilingheight = $ - FRACUNIT*5
                    end
                end
                if mapheaderinfo[gamemap].forceeggwatchtime then
                    eggWatchTimer = tonumber(mapheaderinfo[gamemap].forceeggwatchtime)
                else
                    eggWatchTimer = eggWatchTime.value
                end
            end
        end
        for player in players.iterate do
            if player.mo and player.mo.valid then
                if player.mo.colorized == true
                and player.mo.skincolor == SKINCOLOR_GREY then
                    if player.mo.momz > 0
                    or player.mo.momz < 0
                        P_SetObjectMomZ(player.mo,-FRACUNIT*5,true)
                    end
                    player.mo.friction = 100
                end
                if player.mo.ShadowFriction == true then
                    player.mo.friction = FRACUNIT+(FRACUNIT/4)
					S_ChangeMusic("ESCAPE")
                end
				if player.mo.funniscale != nil
				and player.mo.funniscale != 0
					P_SetScale(player.mo,player.mo.funniscale)
				end
            end
        end
    end
end)

addHook("MapLoad", function()
    if mapheaderinfo[gamemap].forceeggwatchtime then
        eggWatchTimer = tonumber(mapheaderinfo[gamemap].forceeggwatchtime)
    else
        eggWatchTimer = eggWatchTime.value
    end
    for skin in skins.iterate() do
        if not string.find(skinstring,skin.name) then
            print("Skin '" .. skin.name .. "' added to Egg Watch skin list")
            table.insert(skinlist,skin.name)
            skinstring = table.concat(skinlist)
        end
    end
end)

hud.add(function(v,player,cam)
    if eggWatchEnabled.value == 1 then
        v.drawString(24,10,eggwatchName,V_6WIDTHSPACE|V_REDMAP|V_HUDTRANS|V_PERPLAYER|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
        if eggWatchTimer == 1 --One second remaining
            v.drawString(24,20,eggWatchTimer .. " SECOND REMAINING",V_6WIDTHSPACE|V_REDMAP|V_HUDTRANS|V_PERPLAYER|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
        elseif eggWatchTimer < 11 --Ten seconds remaining
            v.drawString(24,20,eggWatchTimer .. " SECONDS REMAINING",V_6WIDTHSPACE|V_REDMAP|V_HUDTRANS|V_PERPLAYER|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
        elseif eggWatchTimer < 31 --Thirty seconds remaining
            v.drawString(24,20,eggWatchTimer .. " SECONDS REMAINING",V_6WIDTHSPACE|V_ORANGEMAP|V_HUDTRANS|V_PERPLAYER|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
        elseif eggWatchTimer < 61 --Sixty seconds remaining / One minute remaining
            v.drawString(24,20,eggWatchTimer .. " SECONDS REMAINING",V_6WIDTHSPACE|V_YELLOWMAP|V_HUDTRANS|V_PERPLAYER|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
        elseif eggWatchTimer < 101 --One hundred seconds remaining
            v.drawString(24,20,eggWatchTimer .. " SECONDS REMAINING",V_6WIDTHSPACE|V_PERIDOTMAP|V_HUDTRANS|V_PERPLAYER|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
        else --More than one hundred seconds remaining
            v.drawString(24,20,eggWatchTimer .. " SECONDS REMAINING",V_6WIDTHSPACE|V_GREENMAP|V_HUDTRANS|V_PERPLAYER|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
        end
    end
end)

local skinstringtemp = ""

COM_AddCommand("eggwatch_scanskins",function(player,args)
	CONS_Printf(player,"Scanning for new skins...")
	skinstringtemp = skinstring
	for skin in skins.iterate() do
        if not string.find(skinstring,skin.name) then
            print("Skin '" .. skin.name .. "' added to Egg Watch skin list")
            table.insert(skinlist,skin.name)
            skinstring = table.concat(skinlist)
        end
    end
	if skinstringtemp == skinstring then
		CONS_Printf(player,"No new skins were found.")
	end
	CONS_Printf(player,"Skin list is " .. table.concat(skinlist,", "))
end)