--Script: Melting
--Usage: Makes your character melt. The melt rate is customizable, but it mostly depends on the brightness. Your character dies once they melt entirely.

local meltingEnabled = CV_RegisterVar({
	name = "melting",
	defaultvalue = 0,
	flags = CV_SHOWMODIF|CV_NETVAR,
	PossibleValue = CV_OnOff
})

local meltingRate = CV_RegisterVar({
	name = "melting_factor",
	defaultvalue = 450,
	flags = CV_SHOWMODIF|CV_NETVAR,
	PossibleValue = CV_UNSIGNED
})

local precipColChance = CV_RegisterVar({
	name = "melting_rainchance",
	defaultvalue = 25,
	flags = CV_SHOWMODIF|CV_NETVAR,
	PossibleValue = CV_UNSIGNED
})

addHook("MapLoad",function()
	if meltingEnabled.value == 0 then
		print("\x85Melting is off.")
	else
		print("\x83Melting is on.")
		print("\x82(Melting) Melt factor is one " .. meltingRate.value .. "th of a fracunit per tic")
		print("\x82(Melting) Rain collection chance is 1/" .. precipColChance.value)
	end
end)

addHook("ThinkFrame", function()
	for player in players.iterate do
		if meltingEnabled.value == 1 then
			if player.mo then
				if player.mo.valid then
					if not multiplayer then --Become ice in singleplayer
						player.mo.color = SKINCOLOR_ICY
					end
					player.mo.colorized = true --Become icey
					if not player.exiting then
						player.mo.spriteyscale = $ - FRACUNIT/meltingRate.value --Melt
					end
					player.mo.spritexscale = FRACUNIT + (FRACUNIT - player.mo.spriteyscale)
					player.powers[pw_underwater] = TICRATE*30 --Underwater breathing
					if curWeather == PRECIP_STORM --Reconstitute in the rain
					or curWeather == PRECIP_RAIN 
					or curWeather == PRECIP_SNOW
						if P_RandomChance(FRACUNIT/precipColChance.value)
						and player.mo.subsector.sector.ceilingpic == "F_SKY1" --Don't take shelter
							player.mo.spriteyscale = $ + FRACUNIT/20
						end
					end
					if player.mo.spriteyscale < 50 then
						if player.mo.health != 0 then --Nobody wants to play a game called
							P_KillMobj(player.mo,nil,nil) --Liquid Water Robo Blast 2
						end
					else
						if player.mo.spriteyscale < FRACUNIT/4 then
							player.mo.color = SKINCOLOR_RED
						else
							if not multiplayer then
								player.mo.color = SKINCOLOR_ICY
							else
								player.mo.color = player.skincolor
							end
						end
					end
					if (player.mo.eflags & MFE_TOUCHWATER)
					or (player.mo.eflags & MFE_UNDERWATER)
						player.mo.spriteyscale = $ + FRACUNIT/40
					end
					if player.mo.spriteyscale > FRACUNIT then
						player.mo.spriteyscale = FRACUNIT
					end
				end
			end
		end
	end
end)

addHook("ShouldDamage", function(mo,inflictor,source,damage,damagetype)
	for player in players.iterate do
		if meltingEnabled.value == 1 then
			if mo == player.mo then
				if damagetype == DMG_FIRE then
					player.mo.spriteyscale = $ - (FRACUNIT/meltingRate.value * 2)
					player.mo.color = SKINCOLOR_RED
					return false
				end
			end
		end
	end
end, MT_PLAYER)

local soundRNGa = P_RandomRange(1,5)

addHook("MobjMoveCollide", function(mo, spike) -- Spike breaking by ARJr.
    if mo.valid
    and (mo.type == MT_PLAYER)
	and meltingEnabled.value == 1
		if spike.type == MT_RING
			if (spike.z >= (mo.z - 26*FRACUNIT)) and (spike.z <= (mo.z + mo.height + 24*FRACUNIT))
				P_KillMobj(spike, mo, mo)
				mo.player.rings = $ + 1
				soundRNGa = P_RandomRange(1,5)
				if soundRNGa == 1 then
					S_StartSound(mo,55)
				elseif soundRNGa == 2 then
					S_StartSound(mo,56)
				elseif soundRNGa == 3 then
					S_StartSound(mo,57)
				elseif soundRNGa == 4 then
					S_StartSound(mo,58)
				elseif soundRNGa == 5 then
					S_StartSound(mo,59)
				end
				mo.spriteyscale = $ + FRACUNIT/5
			elseif (spike.flags2 & MF2_OBJECTFLIP)
			and (spike.z <= (mo.z - 26*FRACUNIT)) and (spike.z >= (mo.z + mo.height+ 24*FRACUNIT))
				P_KillMobj(spike, mo, mo)
				mo.player.rings = $ + 1
				soundRNGa = P_RandomRange(1,5)
				if soundRNGa == 1 then
					S_StartSound(mo,55)
				elseif soundRNGa == 2 then
					S_StartSound(mo,56)
				elseif soundRNGa == 3 then
					S_StartSound(mo,57)
				elseif soundRNGa == 4 then
					S_StartSound(mo,58)
				elseif soundRNGa == 5 then
					S_StartSound(mo,59)
				end
				mo.spriteyscale = $ + FRACUNIT/5
			end
		end
	end
end)